MOMStreamDeckPlugin
===================

MOMStreamDeckPlugin is an interface between DADman and StreamDeck. It appears as a MOM (Monitor Operating Module) to DADman. Please download the original plugin from its original location [here](https://www.lukktone.com/download/MOMStreamDeckPlugin.zip) rather than distributing externally, in order to ensure you always have the latest version.

macOS 10.13 or greater is required. The plugin is a universal binary that will work on both Intel and Apple Silicon (although it will always be started under Rosetta, until StreamDeck itself is updated to be a Universal binary).

This software is provided ”as-is” and without support. MOMStreamDeckPlugin is not endorsed or supported by either Digital Audio Denmark (DAD) or Avid. Officially supported devices include the DAD MOM and Avid EuCon controllers.

Luke Howard `<appsupport@lukktone.com>`

February 2022

StreamDeck installation
-----------------------

First, install the plugin by double-clicking the `com.padl.monitoroperatingmodule.streamDeckPlugin` file in the package you were supplied.

The plugin works best with the included profile which is designed to mimic the physical layout of a hardware MOM. Install this by opening the StreamDeck configuration window, selecting _Edit Profiles: Import Profiles_, and opening `MOM.streamDeckProfile`.

## DADman configuration

Once StreamDeck is configured, switch to DADman, and open _Settings: Device List_. You may need to hit Refresh. Select the StreamDeck device and you should see _Unit ready_; the StreamDeck keycaps should change and you should see the text *Ready* in the top left-hand corner.

Next, go to _Settings: Monitor Profile_, select the Mom tab, and right-click _Add a new MOM_, assigning the StreamDeck. Right-click on the newly created MOM and select _Default preset_. That is a starting point you can customise further. Note that although you can freely any button to any action, MOMStreamDeckPlugin has no way of validating these assignments. 

## Further configuration

The default volume increment is in steps of 0.5dB. This can be changed from the StreamDeck configuration window, when either the volume increment or decrement action is selected. The minimum step is 0.5dB and the maximum is 3.0dB. The MOM cannot display an attenuation greater than -40dB.

Before the connection to DADman is open, the "external" action shows the connection status; when it is open, it shows the volume and device ID. The action also doubles as the "external" button,which on a hardware MOM represents the footswitch input. Because of protocol limitations, it is not possible for this action to change background colour when selected.

The Microsoft Surface Dial is supported as a rotary encoder for controlling volume. The Dial must first be paired with macOS, and then enabled from the Radial Controllers configuration of the plugin's StreamDeck settings (available on the external and volume increment/decrement action settings). Please contact the author if you have a different USB rotary encoder that you would like to see support for.

## Multiple MOM support

There are two advanced modes of operation that allow, respectively, each StreamDeck to appear as a separate MOM, or multiple MOMs to be present on a single StreamDeck. At present they can only be configured from the command line, as the configuration must be enabled before StreamDeck is started (and a restart is required on change).

Before configuring this, you need to have as many spare network interfaces as you intend to configure MOMs. The simplest way to do this is by opening the Network panel in System Preferences, and adding a new interface alias for the builtin Ethernet interface (+ sign in bottom left-hand corner). Give the service a name that reflects it intended usage, and assign it a static IP address that does not conflict with an existing address. The alias subnet mask should be 255.255.255.255. You should also assign a static IP address to the unaliased interface (rather than using DHCP), to ensure it is the first IP address listed. This avoids some issues with Dante and EuCon.

There is also support for linking layers so that layer changes initiated from StreamDeck are synchronised across MOMs.

### Device Router

The first mode, Device Router mode, enables each StreamDeck to appear as a separate MOM. That is, there is a 1:1 relationship between StreamDeck devices and MOMs presented to DADman. This is relatively straightforward to setup as any action you create on a device will be assigned to the MOM associated with that device.

Enable this mode by quitting StreamDeck, opening Terminal, and executing the following command:

```
defaults write com.padl.monitoroperatingmodule.sdPlugin EnablePluginRouter -int 1
```

Restart StreamDeck after doing this. Custom keycaps will need to be re-done and you will need to re-pair devices in DADman, as the MOM serial number is now derived from the StreamDeck device ID rather than the computer. To disable, type the above command, changing `1` to `0`.

### Virtual Device Router

The second mode, Virtual Device Router mode, enables multiple MOMs to be present in a single StreamDeck. You can have as many MOMs as you have spare network interfaces (that are visible by DADman).

Enable this mode by quitting StreamDeck, opening Terminal, and executing the following command:

```
defaults write com.padl.monitoroperatingmodule.sdPlugin EnablePluginRouter -int 2
```

Restart StreamDeck after doing this. MOM configurations from non-router and Device Router modes are can be assigned to virtual MOMs. To disable Virtual Device Router mode, type the above command, changing `2` to `0`.

To finish configuring this mode, you will need to address each action in your StreamDeck and assign it to a MOM using the *Device Link* section. Unassigned actions are in the `Unassigned [Default]` section, which means that they will appear as a MOM on the first available network address. The software tries to remember interface assignments on restart but this is not guaranteed.

## Known Issues

* No profile for the StreamDeck Mini
* Cannot customise action icons because they are always redrawn directly from the MOMStreamDeckPlugin
* If DADman starts before StreamDeck, or the MOM profile is not selected in StreamDeck, it can hang looking for the virtual MOM device
* Only English and German localisations

## Revision History

### MOMStreamDeckPlugin-126

* Fix StreamDeck+ encoder button regression after upgrading to StreamDeck 6.6
* Fix a small memory leak when determining system identifier
* Update to use new common MOM emulation library

### MOMStreamDeckPlugin-125

* Fix StreamDeck+ encoder regression after upgrading to StreamDeck 6.2
* Report SoftMOM version as 3.0 to match iOS/macOS app

### MOMStreamDeckPlugin-124

* Add restore to default button for resetting keycap colors to default values

### MOMStreamDeckPlugin-123

* Add apply to all button for copying keycap colors to all layers

### MOMStreamDeckPlugin-122

* Fix regression in build 120 that broke refreshing keycap labels

### MOMStreamDeckPlugin-121

* Fix regression in build 120 that broke StreamDeck+ support

### MOMStreamDeckPlugin-120

* Support for per-layer customisation of keycap foreground and background colours
* Support for customising keycaps per instantiated action, rather than per MOM. This is an advanced feature enabled by setting the `ContextSpecificKeyCaps` user default to `YES`.
* Include StreamDeck+ sample profile

### MOMStreamDeckPlugin-118

* Make StreamDeck+ encoder display labels for `Cut` and `Dim` larger.

### MOMStreamDeckPlugin-117

* Improved support for StreamDeck+. `Cut` and `Dim` are now shown as icons above the rotary encoder; when both are set, `Cut` is shown with a yellow border. (This functionality is disabled if a custom keycap name has been set elsewhere, in case the buttons are assigned to different functions in DADman.) Pressing the rotary encoder will send a `Cut` message to DADman. Short and long touches on the display send `Dim` and `Ref`, respectively.

### MOMStreamDeckPlugin-115

* Support for StreamDeck+ rotary encoder

### MOMStreamDeckPlugin-114

* Fix regression with StreamDeck 6.0.0 where keycaps would be blank

### MOMStreamDeckPlugin-113

* Allow custom keycap names for Layer button
 
### MOMStreamDeckPlugin-111

* Remove legacy configuration migration: if you have a very old version of MOMStreamDeckPlugin, then you will need to run build 109 to migrate the configuration before updating.
* Allow number of ticks per rotation to be configured for radial controllers:

```
defaults write com.padl.monitoroperatingmodule.sdPlugin TicksPerRotation 20
```

If not set, the default is currently 1 for the Surface Dial and 5 for the nOb Control; this may change in a future release.

### MOMStreamDeckPlugin-109

* Fix a bug where the device ID on external actions was not restored after receiving an Identify message from DADman
* Remove unused UUIDs on network configuration change if they have not yet been assigned to an action

### MOMStreamDeckPlugin-108

* Preliminary support for nOb control
* Send advertisement packet when device ID changes, so other DADman instances are notified
* When Identify is selected from DADman, show device ID in title rather than knob icon
* Show serial numbers rather than UUIDs in Virtual Device Router property inspcetor
* Show network interface address in Virtual Device Router property inspector

### MOMStreamDeckPlugin-100

* Fix bug where custom keycap names could not be removed

### MOMStreamDeckPlugin-97

* Initial public release with plugin router support

### MOMStreamDeckPlugin-86

* Show current volume in larger text
* Better support for IP interface aliases

### MOMStreamDeckPlugin-84

* Allow assignment of Surface Dial button action

### MOMStreamDeckPlugin-80

* Preliminary support for Microsoft Surface Dial as a controller
* Implement identify support (temporarily changes all actions to application icon)
* Report SoftMOM version as 2.1 to match iOS/macOS app
* More internal implementation fixes

### MOMStreamDeckPlugin-79

* Internal fixes

### MOMStreamDeckPlugin-78

* Make StreamDeck discoverable to DADman at startup, before any actions are visible

### MOMStreamDeckPlugin-74

* Allow key caps to be renamed

### MOMStreamDeckPlugin-72

* German localization
* Internal fixes/cleanups

### MOMStreamDeckPlugin-71

* Don't require DADman to be running on the same machine as StreamDeck
* Show selected layer in Layer action key caps
* Ley caps are capitalised instead of all caps
* Support for High Sierra, requires [Swift 5 runtime](https://support.apple.com/kb/dl1998)
 
### MOMStreamDeckPlugin-66

* Initial testing release with wider distribution

## License

Copyright (c) 2018-2022 PADL Software Pty Ltd dba Lukktone. All rights reserved.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
